function [icaEEG, opt] = RemoveStrongArtifacts(icaEEG, Comp, Kthr, F)
%
% This function denoise high amplitude artifacts (e.g. ocular) and remove them from the
% Independent Components (ICs).
%
% INPUT:
%
% icaEEG - matrix of ICA components (Nchanel x Nobservations)
%
% Comp   - # of ICs to be denoised and cleaned (can be a vector)
%
% Kthr   - threshold (multiplayer) for denoising of artifacts
%          (default Kthr = 1.15)
%
% F      - acquisition frequency
%          (default F = 256 Hz)
%
% OUTPUT:
% 
% icaEEG - matrix of cleaned indenpendent components
%
% opt    - vector of threshold values used for filtering of corresponding
%          ICs
%
% NOTE: If a component has no artifacts of a relatively high amplitude
%       the function will skip this component (no action), dispaly a
%       warning and the corresponding output "opt" will be set to zero.
%
% Valeri A. Makarov, vmakarov@opt.ucm.es
% ver 0.1 Sept. 2005
% ver 0.2 May   2006 
%

if nargin < 2, 
    disp('At least two arguments are required!');
    help RemoveStrongArtifacts
    return;
end
if nargin < 3 || isempty(Kthr), Kthr = 1.15; end
if nargin < 4 || isempty(F), F = 256; end
L = round(F*0.1); 
[Nchan, Nobser] = size(icaEEG);
if Nchan > Nobser, 
    error('Problem with data orientation, try to transpose the matrix!'); 
end
N = 2^floor(log2(Nobser)); 
h = daubcqf(6);
opt = zeros(1,length(Comp));
for c=1:length(Comp),
    Y = icaEEG(Comp(c),1:N);
    Sig = median(abs(Y)/0.6745);
    Thr = 4*Sig;
    idx = find(abs(Y) > Thr);
    idx_ext = zeros(1,length(idx)*(2*L+1));
    for k=1:length(idx),
        idx_ext((2*L+1)*(k-1)+1:(2*L+1)*k) = [idx(k)-L:idx(k)+L];
    end
    id_noise=setdiff((1:N), idx_ext);
    id_artef=setdiff((1:N), id_noise);    
    if isempty(id_artef),
        disp(['The component #' num2str(Comp(c)) ' has passed unchaneged']);
        continue;
    end
    thld = 3.6;
    KK = 100;
    LL = floor(log2(length(Y)));
    [xl, xh] = mrdwt(Y, h, LL);
    while KK > Kthr,
        thld = thld + 0.5;
        xh = HardTh(xh, thld);
        xd = mirdwt(xl,xh,h,LL); 
        xn = Y - xd;
        cn=corrcoef(Y(id_noise),xn(id_noise));
        ca=corrcoef(Y(id_artef),xd(id_artef));
        KK = ca(1,2)/cn(1,2);   
    end
    opt(c) = thld;
    Y = icaEEG(Comp(c),end-N+1:end);
    icaEEG(Comp(c),1:N) = xn;
    LL = floor(log2(length(Y)));
    [xl, xh] = mrdwt(Y, h, LL);
    xh = HardTh(xh, thld);
    xd = mirdwt(xl,xh,h,LL); 
    xn = Y - xd; 
    icaEEG(Comp(c),N+1:end) = xn(end-(Nobser-N)+1:end); 
    disp(['The component #' num2str(Comp(c)) ' has been filtered']);
end
